import numpy as np

def hyperbolicPos(Bx,Cx,Cy,Ta,Tb,Tc,z,Vs):
	x=[0,0]
	y=[1,1]	
	Tab=Ta-Tb
	Tac=Ta-Tc
	#print "timediff: ",Tab, "and", Tac
	Rab=Tab*Vs
	Rac=Tac*Vs
	c=np.sqrt(Cx**2+Cy**2)
	if(Rab==0 and Rac==0):
		x=Bx/2.0
		y=-(Cx/Cy)*x+(Cx**2.0+Cy**2.0)/(2.0*Cy)
		y=[y,y]
		x=[x,x]
	elif(Rab==0):
		x=[Bx/2.0,Bx/2]		
		i=Rac**2.0-c**2.0+2.0*Cx*x[0]
		polyeq=[4.0*Cy**2.0-4.0*Rac**2,4.0*Cy*i,i**2.0-(4.0*Rac**2.0)*(z**2.0+x[0]**2.0)]
		y=np.roots(polyeq)
	elif(Rac==0):
		k=(Cx**2.0+Cy**2.0)/(2.0*Cy)
		l=-2.0*(Cx/Cy)*k
		j=Rab**2.0-Bx**2.0
		polyeq=[4.0*Bx**2-4.0*(Rab**2.0)*(1.0+(Cx/Cy)**2.0),4.0*Bx*j-4.0*(Rab**2.0)*l,j**2.0-(4.0*Rab**2.0)*(z**2.0+k**2.0)]
		x=np.roots(polyeq)
		y=[0,0]		
		y[0]=-(Cx/Cy)*x[0]+(Cx**2.0+Cy**2.0)/(2.0*Cy)
		y[1]=-(Cx/Cy)*x[1]+(Cx**2.0+Cy**2.0)/(2.0*Cy)
	else:
		g=(Rac*(Bx/(Rab))-Cx)/(Cy)
		d=-(1.0-(Bx/(Rab))**2.0+g**2.0)
		h=(c**2.0-Rac**2.0+Rac*Rab*(1.0-(Bx/(Rab))**2.0))/(2.0*Cy)
		f=((Rab**2.0)/4.0)*((1.0-(Bx/(Rab))**2.0)**2.0)-h**2.0
		e=Bx*(1.0-(Bx/(Rab))**2.0)-2.0*g*h
		polyeq = [d,e,f-z**2.0]
		x=np.roots(polyeq)
		y=[g*x[0]+h,g*x[1]+h]
	if(np.imag(x[0])!=0 or np.imag(x[1])!=0 or np.imag(y[0])!=0 or np.imag(y[1])!=0):
		return
	return [x,y]


def rotateAroundOrigin(point, angle):
	x=float(point[0])
	y=float(point[1])
	angle=float(angle)
	new_x = x*np.cos(angle)-y*np.sin(angle)
	new_y = x*np.sin(angle)+y*np.cos(angle)
	return [new_x,new_y]

def transformTriangle(posA,posB,posC):
	#translational movement
	new_posA=[0.0,0.0]
	new_posB=[posB[0]-posA[0],posB[1]-posA[1]]
	new_posC=[posC[0]-posA[0],posC[1]-posA[1]]
	angle = 0
	if(new_posB[1]!=0):
		#rotational movement
		angle = -np.arctan(float(new_posB[1])/new_posB[0])
		new_posB=rotateAroundOrigin(new_posB,angle)
		new_posC=rotateAroundOrigin(new_posC,angle)
	return [new_posA,new_posB,new_posC, posA, angle]


def transformAndPosition(recvNode1,recvNode2,recvNode3, Ta, Tb, Tc, Vs, z):
	try:
		transdata= transformTriangle(recvNode1,recvNode2,recvNode3)
		position = hyperbolicPos(transdata[1][0],transdata[2][0],transdata[2][1],Ta,Tb,Tc,z,Vs)

		positions = [[position[0][0],position[1][0]],[position[0][1],position[1][1]]]
		
		positions[0]=rotateAroundOrigin(positions[0],-transdata[4])
		positions[1]=rotateAroundOrigin(positions[1],-transdata[4])
				
		positions[0][0]=positions[0][0]+transdata[3][0]
		positions[1][0]=positions[1][0]+transdata[3][0]
		positions[0][1]=positions[0][1]+transdata[3][1]
		positions[1][1]=positions[1][1]+transdata[3][1]
		
		return positions
	except:
		print "Imaginary value encountered"	

	return


def findDistance(point1,point2):
	return np.sqrt((point1[0]-point2[0])**2+(point1[1]-point2[1])**2)

def findShortestDistance(point,pointSet):
	dist_a = findDistance(point,pointSet[0])
	dist_b = findDistance(point,pointSet[1])
	if(dist_a>dist_b):
		return dist_a
	else:
		return dist_b




def redundantPosition(recvNode1,recvNode2,recvNode3, recvNode4, Ta, Tb, Tc, Td, Vs, z):
	
	#Finding candidate positions in main triangle
	main = transformAndPosition(recvNode1,recvNode2,recvNode3, Ta, Tb, Tc, Vs, z)

	redundancies = 0
	
	try:
		cand_1=main[0]	
		cand_2=main[1]
		cand_1_dist=0
		cand_2_dist=0

		if((findDistance(cand_1,recvNode1)<findDistance(cand_1,recvNode2)) and (findDistance(cand_2,recvNode2)<findDistance(cand_2,recvNode1))):
			if(Ta<Tb):
				return cand_1;
			else:
				return cand_2;
		elif((findDistance(cand_2,recvNode1)<findDistance(cand_2,recvNode2)) and (findDistance(cand_1,recvNode2)<findDistance(cand_1,recvNode1))):
			if(Ta<Tb):
				return cand_2;
			else:
				return cand_1;
		elif((findDistance(cand_1,recvNode1)<findDistance(cand_1,recvNode3)) and (findDistance(cand_2,recvNode3)<findDistance(cand_2,recvNode1))):
			if(Ta<Tc):
				return cand_1;
			else:
				return cand_2;
		elif((findDistance(cand_2,recvNode1)<findDistance(cand_2,recvNode3)) and (findDistance(cand_1,recvNode3)<findDistance(cand_1,recvNode1))):
			if(Ta<Tc):
				return cand_2;
			else:
				return cand_1;

		tri1 = transformAndPosition(recvNode1,recvNode2,recvNode4, Ta, Tb, Td, Vs, z)
		if(tri1):
			cand_1_dist += findShortestDistance(cand_1,tri1)
			cand_2_dist += findShortestDistance(cand_2,tri1)
			redundancies += 1
		
		tri2 = transformAndPosition(recvNode1,recvNode3,recvNode4, Ta, Tc, Td, Vs, z)
		if(tri2):
			cand_1_dist += findShortestDistance(cand_1,tri2)
			cand_2_dist += findShortestDistance(cand_2,tri2)
			redundancies += 1

		tri3 = transformAndPosition(recvNode2,recvNode3,recvNode4, Tb, Tc, Td, Vs, z)
		if(tri3):
			cand_1_dist += findShortestDistance(cand_1,tri3)
			cand_2_dist += findShortestDistance(cand_2,tri3)
			redundancies += 1	
		print "reached end"
		if(redundancies==0):
			print "No pos eligible receptions on middle node"	
			if(findDistance(cand_1,recvNode4)<findDistance(cand_2,recvNode4)):
				return cand_1
		
			else:
				return cand_2	
		
		elif(cand_1_dist<cand_2_dist):
			print "redundancy used1"	
			return cand_1
		else:
			print "redundancy used2"
			return cand_2
	except:
		print "No position found"

















































